-- FishermansFriend is based upon FishingAce

local fmfButton -- our secure button
local fishingPoles = {
	[19022] = true, -- Nat Pagle's Extreme Angler FC-5000
	[19970] = true, -- arcanite fishing pole
	[25978] = true, -- seth's graphite fishing pole
	[6367]  = true, -- big iron fishing pole
	[12225] = true, -- blump family fishing pole
	[6366] = true, -- darkwood fishing pole
	[6256] = true, -- fishing pole
	[6365] = true, -- strong fishing pole
}

local fishingLures = {
   [6533] = { -- "Aquadynamic Fish Attractor",       -- 100 for 10 mins
      ["b"] = 100, -- +100 fishing
      ["s"] = 100, -- requires 100 fishing
   },
   [7307] = { -- "Flesh Eating Worm",                -- 75 for 10 mins
      ["b"] = 75, -- +75 fishing
      ["s"] = 100, -- requires 100 fishing
   },
   [6532] = { -- "Bright Baubles",                   -- 75 for 10 mins
      ["b"] = 75, -- +75 fishing
      ["s"] = 100, -- requires 100 fishing
   },
   [6811] = { -- "Aquadynamic Fish Lens",            -- 50 for 10 mins
      ["b"] = 50, -- +50 fishing
      ["s"] = 50, -- requires 50 fishing
   },
   [6530] = { -- "Nightcrawlers",                    -- 50 for 10 mins
      ["b"] = 50, -- +50 fishing
      ["s"] = 50, -- requires 50 fishing
   },
   [6529] = { -- "Shiny Bauble",                     -- 25 for 10 mins
      ["b"] = 25, -- +25 fishing
      ["s"] = 1, -- requires 1 fishing
   },
}

local FISHINGTEXTURE = "Interface\\Icons\\Trade_Fishing"
local db -- our saved variables database
local overrideOn = false -- do we have the override click active?
local fishingName = ""		-- fishing skill name, will be filled from spellbook
local fishingSubName = ""	-- fishing skill subname, will be filled from spellbook
local isFishing = nil		-- flag if we're in fishing mode or not
local fishingSkill = 0		-- fishing skill level, will be filled from the skill tab
local addingLure = nil		-- are we in the process of adding a lure flag
local currentLure = nil		-- will contain the name of the current lure being added
local resetClickToMove = nil -- will be set to true if we have clicktomove overridden
local resetAutoLoot = nil	-- will be set to true if we have overriden autoloot
local ACTIONDOUBLEWAIT = 0.4 -- seconds to wait for the next click for doubleclicking
local lastClickTime = nil -- used to store the last click time for doubleclick detection
-- saved variable defaults
local defaults = {
	profile = {
		sounds = true,
		autoloot = true,
		autolure = true,
		soundVars = { 
			["Sound_MasterVolume"] = true,
			["Sound_MusicVolume"] = true,
			["Sound_AmbienceVolume"] = true,
			["Sound_SFXVolume"] = true,
			["MapWaterSounds"] = true,
		},
	}
}

local L = LibStub("AceLocale-3.0"):GetLocale("FishermansFriend")

local function giveOptions()
	-- GUI options table
	local options = {
		type = "group",
		icon = FISHINGTEXTURE,
		name = L["Fishermans Friend"],
		get = function( k ) return db[k.arg] end,
		set = function( k, v ) db[k.arg] = v end,
		args = {
			fishdesc = {
				type = "description", order = 0,
				name = L["Fishermans Friend will automatically detect if you are wielding a fishing rod. When you are you can double rightclick to cast your line. You can toggle the features of Fishermans Friend below."],
			},
			autoloot = {
				name = L["Auto Loot"],
				desc = L["Turn on Auto Loot when fishing."],
				type = "toggle",
				arg = "autoloot",
				order = 1, width = "full",
			},
			autolure = {
				name = L["Auto Lure"],
				desc = L["Automatically apply the best lure to your fishing pole."],
				type = "toggle",
				arg = "autolure",
				order = 2, width = "full",
			},
			sounds = {
				name = L["Enhance Sounds"],
				desc = L["Enhance sounds for fishing, lowering all sounds but the bobber (and potentially other sound effects)."],
				type = "toggle",
				arg = "sounds",
				order = 3, width = "full",
			},
		}
	}
	return options
end

-- Addon Declaration
FishermansFriend = LibStub("AceAddon-3.0"):NewAddon("FishermansFriend", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0")

-- Addon Initialization
function FishermansFriend:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("FishermansFriendDB", defaults, "global")
	db = self.db.profile
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Fishermans Friend", giveOptions)
	if GetSpellInfo then
		LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Fishermans Friend", L["Fishermans Friend"])
	else
		self:RegisterChatCommand("fmf", function() LibStub("AceConfigDialog-3.0"):Open("Fishermans Friend") end )	
	end	
	
end

function FishermansFriend:OnEnable()
	-- nil out the following 3 vars for proper reset
	isFishing = nil
	addingLure = nil
	currentLure = nil
	
	self:CreateButton()
	self:HookScript(WorldFrame, "OnMouseDown", "WorldFrameOnMouseDown")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("SPELLS_CHANGED")
	self:RegisterEvent("PLAYER_LOGOUT", "StopFishingMode")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "SPELLS_CHANGED")
	self:RegisterEvent("SKILL_LINES_CHANGED", "SPELLS_CHANGED")
	-- intial setup
	self:UNIT_INVENTORY_CHANGED( "", "player" ) -- find the fishing pole
	self:SPELLS_CHANGED() -- find the fishing skill
end

-- Make sure we stop fishing OnDisable to restore old values of mouselook etc.
function FishermansFriend:OnDisable()
	self:StopFishingMode()
end

-- Remove the overridebinding click after clicking or after setting a lure
local function CleanupOverride()
	local wait = false
	if InCombatLockdown() then
		wait = true
	elseif overrideOn then
		ClearOverrideBindings(fmfButton)
		overrideOn = false		
	end

	if addingLure then
		local sp, rk, dn, ic, st, et = UnitCastingInfo("player")
		if not sp or (dn and dn ~= currentLure ) then
			addingLure = nil
		else
			wait = true
		end
	end
	
	if wait then
		FishermansFriend:ScheduleTimer( CleanupOverride, 1 )
	end
end

function FishermansFriend:CreateButton()
	-- button already exists
	if fmfButton then return end
	
	fmfButton = CreateFrame("CheckButton", "FishermansFriendButton", UIParent, "ActionBarButtonTemplate")
	fmfButton:SetScript("OnAttributeChanged", ActionButton_Update)
	-- place ouside frame
	fmfButton:SetPoint("LEFT", UIParent, "RIGHT", 10000, 0)
	fmfButton:EnableMouse(true)
	fmfButton:RegisterForClicks("RightButtonUp")
	fmfButton:Hide()
	fmfButton:SetScript("PostClick", CleanupOverride)
end

local function CheckForDoubleClick()
	if lastClickTime then
		local pressTime = GetTime()
		local doubleTime = pressTime - lastClickTime
		lastClickTime = pressTime
		if ( doubleTime < ACTIONDOUBLEWAIT ) then
			return true
		end
	end
	lastClickTime = GetTime()
	return false
end

function FishermansFriend:WorldFrameOnMouseDown( this, button )
	if button == "RightButton" and not InCombatLockdown() and 
		isFishing and fishingName and CheckForDoubleClick() then		
			-- We're stealing the mouse-up event, make sure we exit MouseLook
			if IsMouselooking() then MouselookStop() end
			if not self:AddLure() then
				fmfButton:SetAttribute("type", "spell")
				fmfButton:SetAttribute("spell", fishingName)
			end
			SetOverrideBindingClick(fmfButton, true, "BUTTON2", "FishermansFriendButton")
			overrideOn = true
	end
end

function FishermansFriend:UNIT_INVENTORY_CHANGED( event, unit )
	if unit ~= "player" then return end -- we only care about the player
	local slot = GetInventorySlotInfo("MainHandSlot")
	local _,_,itemid = string.find(GetInventoryItemLink("player",slot) or "", "item:(%d+):%d+:%d+:%d+")
	itemid = tonumber(itemid)
	local usingFishingPole = ( itemid and fishingPoles[itemid] )
	-- enable / disable fishing mode only if we have to
	if not isFishing and usingFishingPole then
		self:StartFishingMode()
	elseif isFishing and not usingFishingPole then 
		self:StopFishingMode()
	end
end

function FishermansFriend:SPELLS_CHANGED()
	-- find the fishing skill if possible
	local id = 1
	local spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL)
	while spellTexture do
		if spellTexture == FISHINGTEXTURE then
			fishingName, fishingSubName = GetSpellName(id, BOOKTYPE_SPELL)
			-- found fishing, now get the skill level
			local numskills = GetNumSkillLines()
			for i = 1, numskills do
				local name, _, _, rank, _, modifier = GetSkillLineInfo(i)
				if name == fishingName then
					fishingSkill = rank + modifier
					break
				end
			end	
			fishingName = fishingName.."("..fishingSubName..")"
			break
		end
		id = id + 1
		spellTexture = GetSpellTexture(id, BOOKTYPE_SPELL)
	end
end

function FishermansFriend:GetBestLure()
	local itemid

	local lbag, lslot, lure, lname = nil, nil, nil, nil
	local bonus = 0

	for bag = 0, 4, 1 do
		for slot = 1, GetContainerNumSlots(bag), 1 do
			_,_,itemid,_,itemname = string.find(GetContainerItemLink(bag,slot) or "", "item:(%d+):%d+:%d+:%d(.*)%[(.*)%]")
			itemid = tonumber(itemid)
			if itemid and fishingLures[itemid] then
				lure = fishingLures[itemid]
				if lure.b > bonus and lure.s <= fishingSkill then
					lbag = bag
					lslot = slot
					lname = itemname
					bonus = lure.b
				end
			end
		end
	end
	return lbag, lslot, lname
end

function FishermansFriend:AddLure()
	if not addingLure and db.autolure then
		-- if the pole has an enchantment, we can assume it's got a lure on it (so far, anyway)
		local hmhe = GetWeaponEnchantInfo()
		if not hmhe then
			local tb, ts, name = self:GetBestLure()
			if tb then
				fmfButton:SetAttribute("type", "item")
				fmfButton:SetAttribute("bag", tb)
				fmfButton:SetAttribute("slot", ts)
				local slot = GetInventorySlotInfo("MainHandSlot")
				fmfButton:SetAttribute("target-slot", slot)
				fmfButton:SetAttribute("spell", nil)
				addingLure = true
				currentLure = name
				return true
			end
		end
	end
	currentLure = nil
	return false	
end

function FishermansFriend:StartFishingMode()
	if isFishing then return end
	
	-- Disable Click-to-Move if we're fishing
	if GetCVar("autointeract") == "1" then
		resetClickToMove = true
		SetCVar("autointeract", "0")
	end
	-- autoloot on when fishing
	if db.autoloot and not GetAutoLootDefault() then
		resetAutoLoot = true
		SetAutoLootDefault(true)
	end
	-- sound enhancement
	if db.sounds then self:EnhanceSound() end
	isFishing = true
end

function FishermansFriend:StopFishingMode()
	if not isFishing then return end
	
	if resetClickToMove then
		-- Re-enable Click-to-Move if we changed it
		SetCVar("autointeract", "1")
		resetClickToMove = nil
	end
	if resetAutoLoot then
		SetAutoLootDefault(false)
		resetAutoLoot = nil
	end
	self:RestoreSound()
	isFishing = false
end

function FishermansFriend:EnhanceSound()
	if db.isEnhanced then return end
	for k, v in pairs( db.soundVars ) do
		db.soundVars[k] = tonumber(GetCVar(k))
	end
	SetCVar("Sound_MasterVolume", 1.0)
	SetCVar("Sound_SFXVolume", 1.0)
	SetCVar("Sound_MusicVolume", 0.0)
	SetCVar("Sound_AmbienceVolume", 0.0)
	SetCVar("MapWaterSounds", 0)
	db.isEnhanced = true
end

function FishermansFriend:RestoreSound()
	if not db.isEnhanced then return end
	for k, v in pairs( db.soundVars ) do
		SetCVar(k, v)
		db.soundVars[k] = true
	end
	db.isEnhanced = nil
end

